# SPDX-FileCopyrightText: 2021 Melissa LeBlanc-Williams for Adafruit Industries
#
# SPDX-License-Identifier: MIT
"""Pin definitions for the Orange Pi 3."""

from adafruit_blinka.microcontroller.allwinner.h6 import pin

PD26 = pin.PD26
SDA = pin.PD26
PD25 = pin.PD25
SCL = pin.PD25
PD22 = pin.PD22
PL2 = pin.PL2
PL3 = pin.PL3
PD24 = pin.PD24
UART3_RX = pin.PD24
PD18 = pin.PD18
PD23 = pin.PD23
UART3_TX = pin.PD23
PL10 = pin.PL10
PD15 = pin.PD15
PD16 = pin.PD16
PH5 = pin.PH5
PH6 = pin.PH6
PD21 = pin.PD21
PH4 = pin.PH4
SPI1_CLK = pin.PH4
PH3 = pin.PH3
SPI1_CS = pin.PH3
PL8 = pin.PL8

SCLK = pin.PH4
MOSI = pin.PH5
MISO = pin.PH6
