# SPDX-FileCopyrightText: 2021 Melissa LeBlanc-Williams for Adafruit Industries
#
# SPDX-License-Identifier: MIT
"""Raspberry Pi Pico pin names"""

from adafruit_blinka.microcontroller.rp2040 import pin

GP0 = pin.GP0
GP1 = pin.GP1
GP2 = pin.GP2
GP3 = pin.GP3
GP4 = pin.GP4
GP5 = pin.GP5
GP6 = pin.GP6
GP7 = pin.GP7
GP8 = pin.GP8
GP9 = pin.GP9
GP10 = pin.GP10
GP11 = pin.GP11
GP12 = pin.GP12
GP13 = pin.GP13
GP14 = pin.GP14
GP15 = pin.GP15
GP16 = pin.GP16
GP17 = pin.GP17
GP18 = pin.GP18
GP19 = pin.GP19
GP20 = pin.GP20
GP21 = pin.GP21
GP22 = pin.GP22
GP23 = pin.GP23
GP24 = pin.GP24
GP25 = pin.GP25
GP26 = pin.GP26
GP27 = pin.GP27
GP28 = pin.GP28
LED = GP25
SMPS_MODE = GP23
VBUS_SENSE = GP24
A0 = GP26
A1 = GP27
A2 = GP28
A3 = pin.GP29
VOLTAGE_MONITOR = pin.GP29
