# SPDX-FileCopyrightText: 2021 Melissa LeBlanc-Williams for Adafruit Industries
#
# SPDX-License-Identifier: MIT
"""Pin definitions for the MicroChip MCP2221"""
from adafruit_blinka.microcontroller.mcp2221 import pin

G0 = pin.G0
G1 = pin.G1
G2 = pin.G2
G3 = pin.G3

SCL = pin.SCL
SDA = pin.SDA
