# SPDX-FileCopyrightText: 2021 Melissa LeBlanc-Williams for Adafruit Industries
#
# SPDX-License-Identifier: MIT
"""Return the current version"""
VERSION = (1, 0, 0)

__version__ = ".".join(map(str, VERSION))
