# SPDX-FileCopyrightText: 2021 Melissa LeBlanc-Williams for Adafruit Industries
#
# SPDX-License-Identifier: MIT
"""Pin definitions for the GreatFET One"""
from adafruit_blinka.microcontroller.nxp_lpc4330 import pin

J1_P3 = pin.J1_P3
J1_P4 = pin.J1_P4
J1_P5 = pin.J1_P5
J1_P6 = pin.J1_P6
J1_P7 = pin.J1_P7
J1_P8 = pin.J1_P8
J1_P9 = pin.J1_P9
J1_P10 = pin.J1_P10
J1_P12 = pin.J1_P12
J1_P13 = pin.J1_P13
J1_P14 = pin.J1_P14
J1_P15 = pin.J1_P15
J1_P16 = pin.J1_P16
J1_P17 = pin.J1_P17
J1_P18 = pin.J1_P18
J1_P19 = pin.J1_P19
J1_P20 = pin.J1_P20
J1_P21 = pin.J1_P21
J1_P22 = pin.J1_P22
J1_P23 = pin.J1_P23
J1_P24 = pin.J1_P24
J1_P25 = pin.J1_P25
J1_P26 = pin.J1_P26
J1_P27 = pin.J1_P27
J1_P28 = pin.J1_P28
J1_P29 = pin.J1_P29
J1_P30 = pin.J1_P30
J1_P31 = pin.J1_P31
J1_P32 = pin.J1_P32
J1_P33 = pin.J1_P33
J1_P34 = pin.J1_P34
J1_P35 = pin.J1_P35
J1_P37 = pin.J1_P37
J1_P39 = pin.J1_P39
J1_P40 = pin.J1_P40

# J2 Header Pins
J2_P3 = pin.J2_P3
J2_P4 = pin.J2_P4
J2_P5 = pin.J2_P5
J2_P6 = pin.J2_P6
J2_P7 = pin.J2_P7
J2_P8 = pin.J2_P8
J2_P9 = pin.J2_P9
J2_P10 = pin.J2_P10
J2_P13 = pin.J2_P13
J2_P14 = pin.J2_P14
J2_P15 = pin.J2_P15
J2_P16 = pin.J2_P16
J2_P18 = pin.J2_P18
J2_P19 = pin.J2_P19
J2_P20 = pin.J2_P20
J2_P22 = pin.J2_P22
J2_P23 = pin.J2_P23
J2_P24 = pin.J2_P24
J2_P25 = pin.J2_P25
J2_P27 = pin.J2_P27
J2_P28 = pin.J2_P28
J2_P29 = pin.J2_P29
J2_P30 = pin.J2_P30
J2_P31 = pin.J2_P31
J2_P33 = pin.J2_P33
J2_P34 = pin.J2_P34
J2_P35 = pin.J2_P35
J2_P36 = pin.J2_P36
J2_P37 = pin.J2_P37
J2_P38 = pin.J2_P38

# Bonus Row Pins
J7_P2 = pin.J7_P2
J7_P3 = pin.J7_P3
J7_P6 = pin.J7_P6
J7_P7 = pin.J7_P7
J7_P8 = pin.J7_P8
J7_P13 = pin.J7_P13
J7_P14 = pin.J7_P14
J7_P15 = pin.J7_P15
J7_P16 = pin.J7_P16
J7_P17 = pin.J7_P17
J7_P18 = pin.J7_P18

SDA = pin.SDA
SCL = pin.SCL

SCK = pin.SCK
SCLK = SCK
MOSI = pin.MOSI
MISO = pin.MISO

TX = pin.TX
RX = pin.RX
